#! /usr/bin/env python

'''
[EOS - search for host - on with switch and port host is connected]
Copyright (C) [2008]  [Kamil Derynski]

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>
'''

__author__ = "Kamil Derynski"
__version__ = "0.1.2"

import commands
import sys
import getopt
import os
import time
import atexit
from threading import Thread

debug = False

#SWITCH LIST  (Define here yours switch hostname list)
snlist = ["p103", "p103b", "p11", "p225", "p226", "p226b", "p507", "p510"]
#snlist = ["p226", "pwewe","p507","p103","p90"]
slist = []
test = []
flag = False

#default snlist is empty !
#snlist = []

#FUNCTIONS

def cmd(command):
    u''' Funkcja obslugujaca konsole'''
    result = {}
    output = commands.getstatusoutput(command)
    result["status"] = output[0]
    result["out"] = output[1]
    if (int(result["status"]) != 0):
        raise CmdError(result)
    return result

#--------------------------------------------

def findEther(host):
    ethers = parseEthers("/etc/ethers")
    result = {}

    try:
        key = ethers.get_key(host)
        if not key:
            raise KeyError
        #jesli znaleziono mac adres w ethers
        result["name"] = key[0]
        result["mac"] = host
    except KeyError:
        try:
            #jesli znaleziono nazwe w ethers
            result["name"] = host
            result["mac"] = ethers.get_value(host)
        except KeyError:
            #jesli w ogole nie ma w ethers 
            raise EtherNotFound(result)
    return result

#--------------------------------------------

def macHexToDec(mac):
    u"""Konweruje adres mac z xx:xx:xx:xx:xx:xx do dd.dd.dd.dd.dd.dd""" 
    lMac = str(mac).split(":")
    lMac = [int(val,16) for val in lMac]
    macc = ".".join(["%s" % k for k in lMac])
    return macc

#--------------------------------------------

def find_key(dic, val):
    """return the key of dictionary dic given the value"""
    return [k for k, v in symbol_dic.iteritems() if v == val][0]

def find_value(dic, key):
    """return the value of dictionary dic given the key"""
    return dic[key]

#--------------------------------------------

def parseEthers(file):
    u"""Parsuje plik ethers"""
    f=open(file, 'r')
    rows = f.readlines()
    out = {}
    for val in rows:
        #print val
        if val[0] == "#":
            pass
        else:
            l = val.split()
            if not l:
                pass
            else:
                out[l[1]] = l[0]
    f.close()
    #return out
    return Lookup(out)

#--------------------------------------------

def checkAtEnd():
    if not slist:
        print 'Not found !'
    else:
        for val in slist:
            print val["switch"] + ":" + val["port"]

#CLASSES

class Host(dict):
    u'''Contains information about host'''
    def __init__(self,host=None, mac=None):
        dict.__init__(self)
        if not mac:
            try:
                self["hostname"] = host
                try:
                    self["macHex"] = parseEthers("/etc/ethers")[host]
                except KeyError:
                    cmd("ping -qc1 -t1 " + host)
                    lArp = cmd("arp -n " + host)["out"].split(" ")
                    self["ip"] = lArp[1].strip("()")
                    self["macHex"] = lArp[3]
                self["macInt"] = macHexToDec(self["macHex"])
            except CmdError, e:
                if debug:
                    print 'Error occured with msg:\n ', e.msg["out"]
                sys.exit(2)
        else:
            self["macHex"] = mac
            self["macInt"] = macHexToDec(self["macHex"])

    def showInfo(self):
        print "Host name: " + self["hostname"]
        print "Host IP: " + self["ip"]
        print "Host MAC(Hex): " + self["macHex"]
        print "Host MAC(Int): " + self["macInt"]

    def getMacHex(self):
        return self["mac"]

    def getMacInt(self):
        try:
            return self["macInt"]
        except KeyError:
            sys.exit(2)
            return None

#--------------------------------------------

class SNMPSwitch(dict):
    u'''Switch class '''
    snmpwalk = "snmpwalk -v1 -c public "

    def __init__(self, switch, host):
        u''' Make SNMP request and store information '''
        try:
            self["switch"] = switch
            self["macTable"] = self.getMacTable()
            self["ifNumber"] = self.getIfNumber()
            self["desc"] = self.getDesc()
            self["loc"] = self.getLoc()
            self["port"] = self.searchPort(host)
            self["StS"] = []
        except CmdError:
            snlist.remove(switch)

    def query(self,string):
        return cmd(self.snmpwalk + self["switch"] + " " + string )

    def getMacTable(self):
        oid = ".1.3.6.1.2.1.17.4.3.1.1"
        q = self.query(oid)
        l = q["out"].split("\n")
        i = 0
        for val in l:
            begin = l[i].find("17.4.3.1.1.")
            end = l[i].find(" =")
            l[i] = l[i][begin+11:end]
            i+=1
        return l

    def getIfNumber(self):
        oid = "IF-MIB::ifNumber.0"
        q = self.query(oid)
        begin = q["out"].find("INTEGER: ")
        self["ifNumber"] = q["out"][begin+9:]
        return self["ifNumber"]

    def getDesc(self):
        oid = "SNMPv2-MIB::sysDescr.0"
        q = self.query(oid)
        begin = q["out"].find("STRING: ")
        self["desc"] = q["out"][begin+8:]
        return self["desc"]

    def getLoc(self):
        oid = "SNMPv2-MIB::sysLocation.0"
        q = self.query(oid)
        begin = q["out"].find("STRING: ")
        self["loc"] = q["out"][begin+8:]
        return self["loc"]

    def searchPort(self,hostMac):
        oid = ".1.3.6.1.2.1.17.4.3.1.2." + hostMac
        q = self.query(oid)
        begin = q["out"].find("INTEGER: ")
        if  begin == -1:
            #print "SNMP query error"
            sys.exit(1)
        flag = True
        self["port"] = q["out"][begin+9:]


        if self.checkS(self["port"]):
            flag = False
        elif int(self["port"]) > 0:
            #print "Located @ switch: " + self["switch"] + " on port: " + self["port"]
            #print self["switch"]+":"+self["port"]
            slist.append(self)
            sys.exit(2)
            #print "Switch:      " + self["switch"]
            #print "Description: " + self.getDesc()
            #print "Location:    " + self.getLoc()
            #print "Port:        " + self["port"]
	    #tu poprawimy
	    
        return self["port"]
        #return False

    def portsUse(self):
        oid = "SNMPv2-SMI::mib-2.17.4.3.1.2"
        ports = {}
        l = []

        for val in range(1,int(self.getIfNumber())+1):
            ports[val] = 0

        for val in self.query("SNMPv2-SMI::mib-2.17.4.3.1.2")["out"].split("\n"):
            begin = val.find("INTEGER: ")
            val2 = int(val[begin+9:])
            if val2 > 0:
                ports[val2] = ports[val2]+1

        for val in range(1,int(self.getIfNumber())+1):
            if ports[val] > 1:
                l.append(val)
        return l

    def checkS(self,port):
        l = self.portsUse()
        try:
            l.index(int(port))
        except ValueError:
            return False
        return True

#--------------------------------------------

class CmdError(Exception):
    u'''CmdError exception class '''
    def __init__(self, msg):
        if debug: 
            self.msg = msg

#--------------------------------------------

class EtherNotFound(Exception):
    u'''Exception throw if Key Error ocured '''
    def __init__(self,msg):
        self.msg = msg

#--------------------------------------------

class Lookup(dict):
    '''a dictionary which can lookup value by key, or keys by value'''
    def __init__(self, items=[]):
        """items can be a list of pair_lists or a dictionary"""
        dict.__init__(self, items)

    def get_key(self, value):
        """find the key(s) as a list given a value"""
        return [item[0] for item in self.items() if item[1] == value]

    def get_value(self, key):
        """find the value given a key"""
        return self[key]

#--------------------------------------------

class testit(Thread):
    def __init__ (self,ip):
        Thread.__init__(self)
        self.ip = ip
        self.status = "unknown"
    def run(self):
        try:
            pingaling = cmd("ping -qc1 -t1 " + self.ip)
            self.status = "OK"
        except CmdError:
            snlist.remove(self.ip)
            self.status = "FAIL"

#--------------------------------------------

class Foundit(Thread):
    def __init__ (self,val,mac):
        Thread.__init__(self)
        self.val = val
        self.mac = mac
        self.snmp = ""
        self.status = "unknown"
    def run(self):
        s = SNMPSwitch(self.val,self.mac);
	#slist.append(s)
        #print slist
        self.status = "OK"

#--------------------------------------------

def runMain():

    atexit.register(checkAtEnd)

    #init check - switch list exist
    if not snlist:
        print "Define your switch list in source first !"
        sys.exit(2)

    try:
        arg = sys.argv[1]
    except IndexError:
        print "usage: eos.py host"
        sys.exit(1)

    foundInEthers = {}
    host = {}
    mac = ""

    #check ethers file jesli nie ma pin
    try:
        eth = findEther(arg)
        host = Host(eth["name"],eth["mac"])
    except EtherNotFound, e:
        #print 'Error occured with msg:\n ', e.msg
        #list of all threads
        pinglist = [] 

        #fork-join threading model
        for host in snlist:
           ip = host
           current = testit(ip)
           pinglist.append(current)
           current.start()

        for pingle in pinglist:
           pingle.join()

        host = Host(arg)
        #print host


    mac = host.getMacInt()

    switchlist = []
    pinglist1 = []
    #fork-join threading model
    for val in snlist:
        ip = host
        current = Foundit(val,mac)
        pinglist1.append(current)
        current.start()

    for pingle in switchlist:
        pingle.join()


#--------------------------------------------

if __name__ == '__main__':
    workdir = sys.path[0]
    os.chdir(workdir)
    runMain()
